pub const fn null<T: Thin + ?Sized>() -> User<'static, *const T>Available on crate feature
full only.Expand description
Produces a null User<*const T>
This will always return an error when reading from
pub const fn null<T: Thin + ?Sized>() -> User<'static, *const T>full only.Produces a null User<*const T>
This will always return an error when reading from